#pragma rtGlobals=1		// Use modern global access method.
#pragma hide=1

//////////////////////////////////////////////////////////////////////////////////
//Licence
//////////////////////////////////////////////////////////////////////////////////
//
//	If you use the 'Fit Assistant' to analyze data, please read and cite the following paper:
//	Surace and Interface Analysis, (2014), 46, 505-511
//	This publication contains more details on the fit function used by XPST.
//
//	XPST is a front-end to the IgorPro fitting engine. Although every possible effort was made 
//	to prevent serious bugs and wrong results, the author can not give any warranty for XPST as 
//	large parts actually depend on the underlying IgorPro code. 
//	Author: Dr. Martin Schmid
//
//////////////////////////////////////////////////////////////////////////////////




// This is the 'handle' function for getting the wave overlap
// it calls waveAlign(), which is the command line function that performs the actual task

function WaveOverlap()

string LastGraphName = WinList("*", "", "WIN:")

string Axname = XWaveName(LastGraphName,CsrWave(A))
string Bxname = XWaveName(LastGraphName,CsrWave(B))
string Awave = CsrWave(A)
string Bwave = CsrWave(B)

wave wRef1 = $Awave
wave wRef2 = $Bwave

if ( cmpstr(Axname,"") || cmpstr(Bxname,"") )
	DoAlert 1, "The data are displayed in an X-Y plot. The program requires to merge the current x and y waves to waveform data? Press NO for EXIT."
	//Print "Sorry not implemented yet"
endif

if (V_Flag == 1)
	if (cmpstr(Axname,""))
		wave Ax = $Axname		
		SetScale /I x, Ax[0], Ax[numpnts(Ax)-1], wRef1
	endif
	if (cmpstr(Bxname,""))
		wave Bx = $Bxname		
		SetScale /I x, Bx[0], Bx[numpnts(Bx)-1], wRef2
	endif
endif


// All that stuff  only prevents user errors - maybe Igor has not the most efficient language
strswitch (CsrWave(A))
	case "":       //A not on grah
		strswitch (CsrWave(B) )
			case "":  // B missing as well
				DoAlert 0, "Use both cursors to mark the waves"
				return 1
			default:
				DoAlert 0, "You also have to use cursor A"
				return 1 // A is not there but B
		endswitch
		break
	default:       // A is on graph
		strswitch (CsrWave(B) )
			case "":  // B is missing
				DoAlert 0, "You have also to use cursor B"
				return 1
			default:
				break // A and B are there, proceed
		endswitch
endswitch
		
if ( cmpstr(Awave,Bwave) == 0 )
	DoAlert 0, "Both cursors on the same wave"
	return 1
endif

variable spacingWave1
variable spacingWave2

spacingWave1 = abs( (leftx(wRef1) - rightx(wRef1))/numpnts(wRef1) )
spacingWave2 = abs( (leftx(wRef2) - rightx(wRef2))/numpnts(wRef2) )


//the second wave should be the one with the smaller point distance, so the interpolation is 
//more accurate
string prim
string second

if (spacingWave1 >= spacingWave2)
	prim = Awave
	second = Bwave
else
	prim = Bwave
	second = Awave
endif

// since everything is prepared, call the function itself
WaveOPS(prim, second, "overlap")
end

// Get the values of wave2 at the x-values of wave1
// wave2 should have more datapoints, since those are used to interpolate the values at the
// x values of wave 1 
// the resulting output shows the y characteristic of wave2 at the points of wave1, therefore
// wave1 and the output wave can be directly compared

function WaveOPS(wave1, wave2, task)
string wave1
string wave2   //the wave with the smallest spacing between the xpoints
string task   //either overlap, sum, differnce, product, quotient
// Problem the two waves   (xPrim, yPrim)  and (xSecond, ySecond) should be combined (+, -, *, / )
// due to  xPrim != xSecond   and   #xPrim  !=  # xSecond  a pointwise combination is not possible

// Solution: Calculate the values of ySecond at the locations of xPrim by interpolation and store
// those values to a new wave   (xPrim, ySecondInter)

// then  (xPrim, yPrim)  is combinable with (xPrim, ySecondInter)  since the x values are identical
                                                    
// the y values shall be calculated in the following way by interpolation with a second wave
//	1. Find two points in the second wave with xSecond0 < xPrim < xSecond1
//	2. Get the ordered pairs (xSecond0,ySecond0), (xSecond1, ySecond1)
//	3. Calculate the linear interpolation of between those two points
//	4. Use this equation to calulate the ordered pair (xPrim, ySecondInter)
//	5. Go to the next xPrim

// until no xSecond1 > xPrim can be found
// result

// we start at   (xPrim, yPrim) 
// and get       (xPrim, ySecondInter)    which is an approximaiton for  (xSecond, ySecond)

// the follwoing functions may be useful     
// 			x2pnt   --> get the index which corresponds closest to the x value        

// the following code constructions are necessary:
//	1. reading the cursor, cursors A and B shall mark the two waves at issue, ySecond is automatically
// 	the wave with the smallest x spacing, so the approximation does not get too crude.                                       
//	2. An escape mechanism, if xPrim and xSecond are entirely in different regions, so the condition above
//	can not be fullfilled

wave prim = $wave1
wave second = $wave2
string TargetName
strswitch (task)
	case "overlap":  
		TargetName = "OL_" + wave1+ "_" + wave2
	break
	case "sum":
		TargetName = "SUM_" + wave1+ "_" + wave2
	break
	case "difference":
		TargetName = "DIF_" + wave1+ "_" + wave2
	break
	case "product":
		TargetName = "PRO_" + wave1+ "_" + wave2
	break
	case "quotient":
		TargetName = "QUO_" + wave1+ "_" + wave2
	break
	default:      
endswitch

if ( strlen(TargetName) >= 30)
	doalert 0, "The name(s) of some wave(s) are too long for Igor! Please shorten the names."
	return 1
endif

make /d /n=(numpnts(prim)) /o xPrim
variable xPrimIndex
variable xPrimSpacing

xPrimSpacing = abs( (leftx(prim) - rightx(prim))/numpnts(prim) )


for ( xPrimIndex = 0; xPrimIndex < numpnts(prim); xPrimIndex += 1)
	xPrim[ xPrimIndex ] = min( leftx(prim), rightx(prim)) + xPrimIndex * xPrimSpacing
endfor

// now make a second wave, which will hold the interpolated y values, its start length is zero, everytime
// a point is interpolated it will be attached with redimension
make /d /n=0 /o ySI      //ySecondInter
make /d /n=0 /o xSI      //xSecondInter

// make the waves accessible by generating local references
wave ySIr = ySI
wave xSIr = xSI     

// get variables for the offset and slope of the linear interpolation
variable iUP            // index upper point  x1
variable iLP             // index lower point  x0
variable offset
variable slope

variable ixP    

//now loop through the values of xPrim and check
for ( ixP = 0; ixP < numpnts(prim); ixP += 1)
	// now we are at
	//print xPrim[ ixP ]
	// get the point-index in the second wave, which is closest to this value
	iLP = x2pnt(second,xPrim[ ixP ] )    // returns nothing, if xPrim[ixP] is not in the range of second ... good
	if (iLP <= 0)
		//print "out of range"
		continue
	endif
	
	//print iLP
	iUP = iLP + 1
	if ( iUP > numpnts(second) )
		//print "had to exit"
		continue
	endif
	redimension /n=(numpnts(xSIr) + 1) xSIr
	redimension /n=(numpnts(ySIr) + 1) ySIr
	xSIr [ numpnts(xSIr) ] = xPrim [ixP]
	
	slope = ( second[ iUP ] - second[ iLP ]) / ( pnt2x(second, iUP) - pnt2x(second, iLP) )
	offset = second[ iLP ] - slope * pnt2x(second, iLP)
	
	ySIr [ numpnts(ySIr) ] = offset + slope * xPrim[ ixP ]
endfor

//now we have an ordered pair of xSecondInterpolated and ySecondInterpolated where
// the point spacing is identical to the point spacing in the primary wave, but the y values are
// based on the y-values of the secondary wave.

// still to do: make a duplicate of the primary wave with the same number of points as xSecondInterpolated

duplicate /o /r=(xSIr[0],xSIr[numpnts(xSIr) -1]) prim NewPrime    //this does not say anything about an internal scaling of NewPrime
wave NewPrimeRef = NewPrime
SetScale /I x, xSIr[0],xSIr[numpnts(xSIr) -1], NewPrime
SetScale /I x, xSIr[0],xSIr[numpnts(xSIr) -1], ySIr

duplicate /o ySIr $TargetName
wave OL = $TargetName            //OL  = OverLap

variable iOL                              // iOL  = indexOverLap

strswitch (task)
	case "overlap":  
		for ( iOL = 0; iOL < numpnts(OL); iOL += 1)
			OL[ iOL ] = min( NewPrimeRef[ iOL ], ySIr [ iOL] )
		endfor
	break
	case "sum":
		for ( iOL = 0; iOL < numpnts(OL); iOL += 1)
			OL[ iOL ] =  NewPrimeRef[ iOL ]+ ySIr [ iOL] 
		endfor
	break
	case "difference":
		for ( iOL = 0; iOL < numpnts(OL); iOL += 1)
			OL[ iOL ] =  NewPrimeRef[ iOL ] - ySIr [ iOL] 
		endfor
	break
	case "product":
		for ( iOL = 0; iOL < numpnts(OL); iOL += 1)
			OL[ iOL ] =  NewPrimeRef[ iOL ] * ySIr [ iOL] 
		endfor
	break
	case "quotient":
		for ( iOL = 0; iOL < numpnts(OL); iOL += 1)
			OL[ iOL ] =  NewPrimeRef[ iOL ]/ ySIr [ iOL] 
		endfor
	break
	default:      
endswitch

killwaves /Z xPrim
killwaves /Z ySI, xSI, NewPrime
end